@@ -10,9 +10,9 @@ import os
 import sys
 import struct
 
-import baker
+import mando
+from mando.rst_text_formatter import RSTHelpFormatter
 import pandas as pd
-import six
 
 from tstoolbox import tsutils
 
@@ -27,7 +27,7 @@ interval2codemap = {'yearly': 5,
                     'daily': 3,
                     'bivl': 2}
 
-code2freqmap = {5: 'Y',
+code2freqmap = {5: 'A',
                 4: 'M',
                 3: 'D',
                 2: None}
@@ -71,8 +71,7 @@ def _get_data(binfilename,
               'RCHRES': ['HYDR', 'CONS', 'HTRCH', 'SEDTRN',
                          'GQUAL', 'OXRX', 'NUTRX', 'PLANK',
                          'PHCARB', 'INFLOW', 'OFLOW', 'ROFLOW', ''],
-              '': ['']
-             }
+              '': ['']}
 
     collect_dict = {}
     lablist = []
@@ -93,7 +92,9 @@ def _get_data(binfilename,
 *
 '''.format(label))
 
-        if words[1]:
+        words = [None if i is '' else i for i in words]
+
+        if words[1] is not None:
             words[1] = words[1].upper()
             if words[1] not in testem.keys():
                 raise ValueError('''
@@ -103,7 +104,7 @@ def _get_data(binfilename,
 *
 '''.format(words[1]))
 
-        if words[2]:
+        if words[2] is not None:
             try:
                 words[2] = int(words[2])
                 if words[2] < 1 or words[2] > 999:
@@ -116,7 +117,7 @@ def _get_data(binfilename,
 *
 '''.format(words[2]))
 
-        if words[3]:
+        if words[3] is not None:
             words[3] = words[3].upper()
             if words[3] not in testem[words[1]]:
                 raise ValueError('''
@@ -127,10 +128,6 @@ def _get_data(binfilename,
 *
 '''.format(words[1], testem[words[1]][:-1], words[3]))
 
-        for index in list(range(len(words))):
-            if words[index] is '':
-                words[index] = None
-
         words.append(intervalcode)
         lablist.append(words)
 
@@ -142,38 +139,28 @@ def _get_data(binfilename,
         labeltest = {}
         vnames = {}
         ndates = {}
-        tindex = 0
-        optype_list = [six.b('PERLND'), six.b('IMPLND'), six.b('RCHRES')]
+        rectype = 0
+        fl.read(1)
         while 1:
-            fl.seek(tindex)
-            initial_search = fl.read(25)
-            search_index = [initial_search.find(i) for i in optype_list]
-            maxsindex = max(search_index)
-            search_index_list = [maxsindex
-                                 if i == -1
-                                 else i
-                                 for i in search_index]
-            search_index = min(search_index_list)
-            if search_index == -1:
-                break
-            tindex = tindex + search_index - 4
-            fl.seek(tindex)
-            rectype, optype, lue, section = struct.unpack('I8sI8s',
-                                                          fl.read(24))
-            optype = optype.strip()
-            section = section.strip()
-            lue = int(lue)
-
-            if rectype == 0:
-                fl.seek(tindex - 4)
+            try:
                 reclen1, reclen2, reclen3, reclen = struct.unpack('4B',
                                                                   fl.read(4))
-                reclen1 = int(reclen1/4)
-                reclen2 = reclen2*64 + reclen1
-                reclen3 = reclen3*16384 + reclen2
-                reclen = reclen*4194304 + reclen3
-                fl.seek(tindex + 24)
-                slen = 0
+            except struct.error:
+                break
+            reclen1 = int(reclen1/4)
+            reclen2 = reclen2*64 + reclen1
+            reclen3 = reclen3*16384 + reclen2
+            reclen = reclen*4194304 + reclen3
+            slen = 0
+            rectype, optype, lue, section = struct.unpack('I8sI8s',
+                                                          fl.read(24))
+
+            rectype = int(rectype)
+            lue = int(lue)
+            optype = optype.strip()
+            section = section.strip()
+
+            if rectype == 0:
                 while slen < reclen - 28:
                     length = struct.unpack('I', fl.read(4))[0]
                     slen = slen + length + 4
@@ -185,7 +172,7 @@ def _get_data(binfilename,
                 # Data record
                 numvals = len(vnames[(lue, section)])
 
-                (unit_flag,
+                (_,
                  level,
                  year,
                  month,
@@ -219,9 +206,12 @@ def _get_data(binfilename,
                     else:
                         mindate = min(mindate, ndate)
                         maxdate = max(maxdate, ndate)
-                        collect_dict[tmpkey] = (mindate, maxdate)
-            tindex = fl.tell()
-
+                        pdoffset = code2freqmap[level]
+                        collect_dict[tmpkey[1:]] = (pd.Period(mindate,
+                                                              freq=pdoffset),
+                                                    pd.Period(maxdate,
+                                                              freq=pdoffset))
+            fl.read(2)
 
     if not collect_dict:
         raise ValueError('''
@@ -246,19 +236,22 @@ def _get_data(binfilename,
     return ndates, collect_dict
 
 
-@baker.command
+@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
 def extract(hbnfilename, interval, *labels, **kwds):
-    '''
-    Prints out data to the screen from a HSPF binary output file.
+    '''Prints out data to the screen from a HSPF binary output file.
 
-    :param hbnfilename: The HSPF binary output file
-    :param interval: One of 'yearly', 'monthly', 'daily', or 'BIVL'.
-        The 'BIVL' option is a sub-daily interval defined in the UCI file.
-        Typically 'BIVL' is used for hourly output, but can be set to any
-        value that evenly divides into a day.
-    :param labels: The remaining arguments uniquely identify a time-series
-        in the binary file.  The format is
-        'OPERATIONTYPE,ID,SECTION,VARIABLE'.
+    Parameters
+    ----------
+    hbnfilename
+        The HSPF binary output file
+    interval
+        One of 'yearly', 'monthly', 'daily', or 'BIVL'.  The 'BIVL'
+        option is a sub-daily interval defined in the UCI file.
+        Typically 'BIVL' is used for hourly output, but can be set to
+        any value that evenly divides into a day.
+    labels
+        The remaining arguments uniquely identify a time-series in the
+        binary file.  The format is 'OPERATIONTYPE,ID,SECTION,VARIABLE'.
 
         For example: 'PERLND,101,PWATER,UZS IMPLND,101,IWATER,RETS'
 
@@ -273,13 +266,17 @@ def extract(hbnfilename, interval, *labels, **kwds):
         'PERLND,,,TAET'
 
         Note that there are spaces ONLY between label specifications.
-    :param time_stamp: For the interval defines the location of the time
-        stamp. If set to 'begin', the time stamp is at the begining of the
-        interval.  If set to any other string, the reported time stamp will
-        represent the end of the interval.  Default is 'begin'.  Place after
-        ALL labels.
-    :param sorted: Should ALL columns be sorted?
-        Default is False.  Place after ALL labels.
+    time_stamp
+        [optional, default is 'begin']
+
+        For the interval defines the location of the time stamp. If set to
+        'begin', the time stamp is at the begining of the interval.  If set to
+        any other string, the reported time stamp will represent the end of the
+        interval.  Place after ALL labels.
+    sorted
+        [optional, default is False]
+
+        Should ALL columns be sorted?  Place after ALL labels.
     '''
     try:
         time_stamp = kwds.pop('time_stamp')
@@ -333,56 +330,75 @@ def extract(hbnfilename, interval, *labels, **kwds):
         skeys.sort(key=lambda tup: tup[1:])
     else:
         skeys.sort()
-    for label in skeys:
-        order, ot, lu, sec, vn, lev = label
 
-        tmpres = pd.DataFrame(data[label],
-                              columns=['{0}_{1}_{2}_{3}'.format(
-                                  ot, lu, vn, lev)],
-                              index=index)
-        try:
-            result = result.join(tmpres)
-        except NameError:
-            result = tmpres
+    result = pd.concat([pd.Series(data[i], index=index) for i in skeys],
+                       axis=1, join_axes=[pd.Index(index)])
+
+    columns = ['{0}_{1}_{2}_{3}'.format(i[1], i[2], i[4], i[5]) for i in skeys]
+    result.columns = columns
 
     if time_stamp == 'begin':
-        result = tsutils.asbestfreq(result)[0]
+        result = tsutils.asbestfreq(result)
         result = result.tshift(-1)
 
+    result.index.name = 'Datetime'
+
     return tsutils.printiso(result)
 
 
-@baker.command
-def catalog(hbnfilename):
+@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
+@tsutils.doc(tsutils.docstrings)
+def catalog(hbnfilename, tablefmt='simple', header='default'):
     '''
     Prints out a catalog of data sets in the binary file.
 
-    The first part of each line up to the first space is the label that can
-    be used with the 'extract' command.
+    The first four items on the line can be used with the 'extract'
+    command.
+
+    Parameters
+    ----------
+    hbnfilename
+        The HSPF binary output file.
+    {tablefmt}
+    {header}
 
-    :param hbnfilename: The HSPF binary output file
     '''
+    # PERLND  905  PWATER  SURS  5  1951  2001  yearly
+    # PERLND  905  PWATER  TAET  5  1951  2001  yearly
     catlog = _get_data(hbnfilename, None, [',,,'], catalog_only=True)[1]
     if tsutils.test_cli() is False:
         return catlog
     catkeys = list(catlog.keys())
     catkeys.sort()
+    if header == "default":
+        header = ['LUE', 'LC', 'GROUP', 'VAR', 'TC', 'START', 'END', 'TC']
+    result = []
     for cat in catkeys:
-        print('{0},{1},{2},{3}  ,{5}, {6}, {7}'.format(
-            *(cat[1:] + catlog[cat] +
-              (code2intervalmap[cat[-1]],))))
+        result.append(cat +
+                      catlog[cat] +
+                      (code2intervalmap[cat[-1]],))
+    return tsutils.printiso(result,
+                            tablefmt=tablefmt,
+                            headers=header)
 
 
-@baker.command
+@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
 def dump(hbnfilename, time_stamp='begin'):
     '''
     Prints out ALL data from a HSPF binary output file.
 
-    :param hbnfilename: The HSPF binary output file
-    :param time_stamp: For the interval defines the location of the time
-        stamp. If set to 'begin', the time stamp is at the begining of the
-        interval.  If set to any other string, the reported time stamp will
-        represent the end of the interval.  Default is 'begin'.
+    Parameters
+    ----------
+    hbnfilename
+        The HSPF binary output file
+    time_stamp
+        [optional, default is 'begin']
+
+        For the interval defines the location of the time stamp. If set
+        to 'begin', the time stamp is at the begining of the interval.
+        If set to any other string, the reported time stamp will
+        represent the end of the interval.  Default is 'begin'.Z
+
     '''
     if time_stamp not in ['begin', 'end']:
         raise ValueError('''
@@ -396,34 +412,27 @@ def dump(hbnfilename, time_stamp='begin'):
     skeys = list(data.keys())
     skeys.sort()
 
-    for label in skeys:
-        order, ot, lu, sec, vn, lev = label
-        nindex = list(index[lev].keys())
-        nindex.sort()
+    result = pd.concat([pd.Series(data[i], index=index) for i in skeys],
+                       axis=1, join_axes=[pd.Index(index)])
 
-        tmpres = pd.DataFrame(data[label],
-                              columns=['{0}_{1}_{2}_{3}'.format(
-                                  ot, lu, vn, lev)],
-                              index=nindex)
-        try:
-            result = result.join(tmpres)
-        except NameError:
-            result = tmpres
+    columns = ['{0}_{1}_{2}_{3}'.format(i[1], i[2], i[4], i[5]) for i in skeys]
+    result.columns = columns
 
     if time_stamp == 'begin':
-        result = tsutils.asbestfreq(result)[0]
+        result = tsutils.asbestfreq(result)
         result = result.tshift(-1)
 
-    try:
-        fresult = fresult.join(result)
-    except NameError:
-        fresult = result
-    del result
-
-    return tsutils.printiso(fresult)
+    return tsutils.printiso(result)
 
 
-@baker.command
+@mando.command()
+def about():
+    """Display version number and system information.
+    """
+    tsutils.about(__name__)
+
+
+@mando.command
 def time_series(hbnfilename, interval, *labels, **kwds):
     ''' DEPRECATED: Use 'extract' instead.
     '''
@@ -433,4 +442,8 @@ def time_series(hbnfilename, interval, *labels, **kwds):
 def main():
     if not os.path.exists('debug_hspfbintoolbox'):
         sys.tracebacklimit = 0
-    baker.run()
+    mando.main()
+
+
+if __name__ == '__main__':
+    main()
